%   Script ANALDE.M zur Herstellung von Bildern
%
%   Analyse des Dampferzeugers
%
%  verwendet BSPDE.M, EPSBILD.M
%
% J. Lunze
% 29.9.1996
% fr 3. Auflage: 19.4.2004
% fr 4. Auflage: 11.1.2006
% fr 5. Auflage: 18.10.2007
% fr 7. Auflage 31.3.2012
% fr 10. Auflage: 3.9.2019
%
%
clear
close all

Bspde;

%%
%
%    Berechnung der bergangsfunktionsmatrix
%
%
figure(1)
T=[0:1:150];
subplot(2, 2, 1);
[Y, T, X]=step(Dampferzeuger, T);
plot(T, Y(:,1,1),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0], 'k-', [0 0],[-0.5 2.5],'k:');
axis([-10, 150, -0.5, 2.5]);
xleer;
yticks([0 2]);
latexylabel('$$h_{11}$$');
rotateY;
latextitle('\"Ubergangsfunktionsmatrix');
%
subplot(2, 2, 3);
plot(T, Y(:,2,1),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0],'b', [0 0],[-0.5 0.5],'k:');
axis([-10, 150, -0.4, 0.2]);
latexxlabel('$$t$$ in min');
latexylabel('$$h_{21}$$');
rotateY;
xticks([0 100]);
yticks([-0.4 -0.2 0 0.2]);
%
subplot(2, 2, 2);
plot(T, Y(:,1,2), 'b',[0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 2.5],'k:');
axis([-10, 150, -0.01, 0.04]);
xleer;
latextitle('des Dampferzeugers');
latexylabel('$$h_{12}$$');
rotateY;
yticks([0 0.04]);
%
subplot(2, 2, 4);
plot(T, Y(:,2,2), 'b',[0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 0.5],'k:');
axis([-10, 150, -0.02, 0.1]);
latexxlabel('$$t$$ in min');
latexylabel('$$h_{22}$$');
rotateY;
yticks([0 0.1]);
xticks([0 100]);


%%
%
%   Matrix der statischen Verstrkungsfaktoren
%
%
dcgain(A, B, C, D)

%%
%
%   Berechnung der Gewichtsfunktionsmatrix
%
%
figure(2)
T=[0:1:150];
subplot(2, 2, 1);
[Y, T, X]=impulse(Dampferzeuger, T);
plot(T, Y(:,1,1),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 2.5],'k:');
axis([-10, 150, -0.01, 0.05]);
xleer;
latexylabel('$$g_{11}$$');
rotateY;
yticks([0 0.04]);
latextitle('Gewichtsfunktionsmatrix');
%
subplot(2, 2, 3);
plot(T, Y(:,2,1),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 0.5],'k:');
axis([-10, 150, -0.006, 0.001]);
latexxlabel('$$t$$ in min');
latexylabel('$$g_{21}$$');
rotateY;
yticks([-0.006 0]);
xticks([0 100]);
%
subplot(2, 2, 2);
plot(T, Y(:,1,2),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 2.5],'k:');
axis([-10, 150, -0.0001, 0.00065]);
xleer;
latextitle('des Dampferzeugers');
latexylabel('$$g_{12}$$');
rotateY;
yticks([0 6e-4]);
subplot(2, 2, 4);
plot(T, Y(:,2,2),'b', [0 150], [0 0], 'k:', [-10, 0], [0, 0], 'b', [0 0],[-0.5 0.5],'k:');
axis([-10, 150, -0.0003, 0.0012]);
latexxlabel('$$t$$ in min');
latexylabel('$$g_{22}$$');
rotateY;
yticks([0 0.001]);
xticks([0 100]);
%




%
%     Ausgabe der Bilder als .eps-Dateien
%
Epsbildc15('analde')
