%   Script BSPFUPROJ.M zur Erzeugung von Beispieldaten
%
%    Frequenz-bergabeleistungs-Verhalten eines Elektroenergienetzes
%    (Projektaufgabe)
%
%  Daten in Anlehung der Parameter aus BSPFU.M
%  aber fr Netz mit nur 2 Teilnetzen, von denen jedes nur 1 Kraftwerk und 1 Verbraucher hat
%  (vgl. "Regelungstechnik", Band 2, Kapitel 6)
%
% J. Lunze
% 4.3.2004
% fr 5. Auflage: 18.10.2007
% fr 10. Auflage: 3.9.2019
%
%
clear A B C D E F DE FE E1 E2 Kraftwerk1 Kraftwerk2 r n m
%%
%
%  Modell: 2 gekoppelte Elektroenergienetze
%
%  Eingnge  u_i: Sollwert des Leistungsreglers des Kraftwerks im Netz i
%  Strungen d_i: Lastnderung im Netz i
%  Ausgangsvektor  y: Frequenz f
%                     Generatorleistung p_G1
%                     Generatorleistung p_G2
%                     Verbraucheleistung p_V1
%                     Verbraucherleistung p_V2
%                     bergabeleistung p_t1
%

% Zeit in s
% Frequenz x_1 in mHz
% Leistung in MW
%                          Parameter
T1=3.270;        % MWs/mHz
T2=3.920;        % MWs/mHz
T=T1+T2;
kV1=0.506;        % MW/mHz  Leistungszahl der Verbraucher im Netz 1
kV2=0.430;        % MW/mHz  Leistungszahl der Verbraucher im Netz 2
kV=kV1+kV2;  
TV1=6;             % s
TV2=5.5;           % s
AG1=[0 -0.125 -0.25;
    1.25 -1.875 -1.25;
    0  0.0625 -0.125];
bG1=[0.25; 1.248; 0];
eG1=1.2*[-0.0662; -0.331; 0]; % Faktor 1.2 zur Anpassung der Leistungszahl des Netzes 1
                      % bei Anpassung an Beispiel mit 3 Netzen: 2.49
cG1=[0 0.5 1];    
dG1=0;
AG2=[-0.2 0;
    1 -0.25];
bG2=[0.05; 0];
eG2=1.2*[-0.0167; 0]; % Faktor 1.2 zur Anpassung der Leistungszahl des Netzes
                     % bei Anpassung an Beispiel mit 3 Netzen: 3.37
cG2=[0 1];       
dG2=0;
Kraftwerk1=ss(AG1, eG1, cG1, dG1);   % Kraftwerk 1: Eingang=f,  Ausgang=p_G1
ks1=-dcgain(Kraftwerk1);             % Leistungszahl des Kraftwerks 1
Kraftwerk2=ss(AG2, eG2, cG2, dG2);   % Kraftwerk 2: Eingang=f, Ausgang=p_G2
ks2=-dcgain(Kraftwerk2);             % Leistungszahl des Kraftwerks 2
k1=ks1+kV1;                          % Leistungszahl des Netzes 1
k2=ks2+kV2;                          % Leistungszahl des Netzes 2
%
%   Zustandsraummodell des Gesamtsystems
%   (vgl. Lsung der Projektaufgabe in "Regelungstechnik", Band 2)
n=8;
m=2;
r=6;
A=[0       1/T*cG1     -1/T       1/T*cG2      -1/T;
   eG1     AG1         zeros(3,1) zeros(3,2)   zeros(3,1);
   kV1/TV1 zeros(1,3)  -1/TV1     zeros(1,2)   0;
   eG2     zeros(2,3)  zeros(2,1) AG2          zeros(2,1);
   kV2/TV2 zeros(1,3)  0          zeros(1,2)   -1/TV2];
B=[0            0;
    bG1        zeros(3,1);
    0           0;
    zeros(2,1) bG2; 
    0           0];
E=[-1/T        -1/T;
    zeros(3,1) zeros(3,1);
    0          0;
    zeros(2,1) zeros(2,1);
    0          0];
%  Ausgangsvektor y=(f p_G1 p_G2 p_V1 p_V2 p_t)':  y=C x 
C=[1   zeros(1,3) 0     zeros(1,2)  0;
    0  cG1        0     zeros(1,2)  0;
    0  zeros(1,3) 0     cG2         0;
    0  zeros(1,3) 1     zeros(1,2)  0;
    0  zeros(1,3) 0     zeros(1,2)  1;
    0  T2/T*cG1  -T2/T -T1/T*cG2   T1/T];
D=zeros(r,m);
F=[0      0; 
   0      0;
   0      0;
   0      0;
   0      0; 
   -T2/T T1/T];
%  e=(e1 e2)'  Netzkennlinienfehler:  e=CE x + DE u + FE d
CE=[C(6,:)+k1*C(1,:);
    -C(6,:)+k2*C(1,:)];
DE=zeros(2,2);
FE=[F(6,:);
    -F(6,:)];



E1=E(1:8, 1);                 % in Zustandsgleichung: ...+E1*d1  mit d1=Lastnderung im 1. Teilnetz
E2=E(1:8, 2);
