%   Script DNOTSTB.M zur Herstellung von Bildern
%
%   Verlust der Steuerbarkeit durch Abtastung
%
% verwendet EPSBILD.M
%
% J. Lunze
% fr 5. Auflage: 2.1.2008
% fr 7. Auflage 31.3.2012
% fr 10. Auflage: 9.9.2019
%
%
clear
close all

A=[0 pi; -pi 0];
B=[1; 1];
C=[1 1];
D=0;

System=ss(A, B, C, D);


%%  Zustands\"uberf\"uhrung (0, 0) -> (1, 2) mit st\"uckweise konstantem Eingang
%
%
figure(1);
xe=[1 2]';   % wie bisher
x0=[0 0]';
Te=2.2;     % Endzeit
T1=1.1;       % u(t) = u0 fr 0 < t < T1
            % u(t) = u1 fr T1 < t < Te
p0=-inv(A)*expm(A*(Te-T1))*B +inv(A)*expm(A*Te)*B;
p1=-inv(A)*B +inv(A)*expm(A*(Te-T1))*B;
P=[p0 p1];
U=inv(P)*(xe-(expm(-A*Te)*x0));
u0=U(1,1);
u1=U(2,1);

Th=Te;                   % Simulationshorizont
dt=0.001;
T=[0:dt:Th];
k=0;
U=u0*ones(1,length(T));
k1=T1/dt;
for k=k1:length(T)
    U(1,k)=u1;
end
[Y, T, X]=lsim(System, U, T, x0);
subplot(6,1,[1,2]);
plot(T, X(:,1), 'b', [-0.3 0], [0 0], 'b');
hold on;
plot([0 0], [-10 12], 'k:', [Te Te], [-10 12], 'k:', [-0.3 3 ], [0 0], 'k:');
plot(Te, 1, 'bo');
latextitle('Umsteuerung des Systems $$x_0=(0 0) \rightarrow x_e=(1 2)$$');
xleer;
latexylabel('$$x_1$$');
rotateY;
axis([-0.3 2.7 -2.5 6.5]);
yticks([0 5]);
hold off;
subplot(6,1,[3,4]);
plot(T, X(:,2), 'b', [-0.3 0], [0 0], 'b');
hold on;
plot(Te, 2, 'bo');
plot([0 0], [-10 12], 'k:', [Te Te], [-10 12], 'k:', [-0.3 3 ], [0 0], 'k:');
%text(Te, xe(2,1), 'b*');
xleer;
latexylabel('$$x_2$$');
rotateY;
axis([-0.3 2.7 -6.5 3]);
yticks([-5 0]);
hold off;
subplot(6,1,[5,6]);
plot(T, U, 'b', [-0.3 0 0], [0 0 U(1,1)], 'b');
hold on;
plot([-0.3 3], [0 0], 'k:', [Te Te], [-150 100], 'k:', [0 0], [-150 100], 'k:');
latexylabel('$$u$$');
rotateY;
latexxlabel('$$t$$');
axis([-0.3 2.7 -1 8]);
xticks([0 1 2 3 4]);
yticks([0 5]);
hold off;

%%     Ausgabe der Bilder als .eps-Dateien
%
Epsbildc15('dnotstb');
