%   Script DREALK.M zur Herstellung von Bildern
%
%   zeitdiskrete Realisierung eines P-Reglers
%
% verwendet EPSBILD.M
%
% J. Lunze
% 8.1.1997
% fr 2. Auflage: 15.12.2001
% fr 3. Auflage: 19.4.2004
% fr 5. Auflage: 18.10.2007
% fr 7. Auflage 25.5.2012
% fr 10. Auflage: 9.9.2019
%
%
%
clear
close all
%         kontinuierlicher Regler
s1=-1;
s2=-3;
zk=1;
nk=poly([s1, s2]);
%         diskreter Regler
Ta=0.05;
zd=[1 1];
z1=exp(s1*Ta);
z2=exp(s2*Ta);
nd=poly([z1, z2]);
zd=zd/ddcgain(zd, nd)*dcgain(zk, nk);

%%
%
%     Vergleich der Bodediagramme des kontinuierlichen Reglers
%     mit der zeitdiskreten Realisierung
%
%
figure(1)
[Betragk, Phasek, Wk]=bode(zk, nk);
[Betragd, Phased, Wd]=dbode(zd, nd, Ta);
subplot(4,1,[1,2]);
semilogx(Wk, 20*log10(Betragk),'b--', Wd, 20*log10(Betragd), 'b');
latextitle('Vergleich des kontinuierlichen und zeitdiskreten Reglers');
xleer;
grid on;
yticks([-40 0]);
latexylabel('$$|K|$$ in dB');
rotateY;
axis([0.01 10 -50 0]);
subplot(4,1,[3,4]);
semilogx(Wk, Phasek, 'b--', Wd, Phased, 'b');
latexxlabel('$$\omega$$ in $${\rm rad/s}$$'); %  in $$\frac{\rm rad}{\rm s}$$
yticks([-200 0]);
latexylabel('$${\rm arg}(K)$$');
rotateY;
grid on;
axis([0.01 10 -200 0]);
xticks([0.01 0.1 1 10 100]);


%
%       Herstellung von Bild-Dateien
%
Epsbildc15('drealk');
